////////////////////////////////////////////////////
////////////////////////////////////////////////////
// !!! this is not public domain !!!
// You can do what you want if it never hurt bonito.
// Otherwise, you will need the consent of Bonito.	
////////////////////////////////////////////////////
// RadioJet Open Source Project ExtIO_RJ10
// www.bonito.net/radiojet/opensource
////////////////////////////////////////////////////
////////////////////////////////////////////////////
#include "stdafx.h"
#include "GlobalTools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

typedef  BOOL ( *DLL_rmDeviceStart)(WORD wVID, WORD wPID, HWND hWnd,void* pHIDCallbackProc, DWORD DeviceAddedMsg, DWORD DeviceRemovedMsg);
typedef  BOOL ( *DLL_rmIsDevicePresent)(void);
typedef  BOOL ( *DLL_rmDeviceClose)(void);
typedef  void ( *DLL_rmHandleDeviceChangeNotification)(DWORD wParam, DWORD lParam);
typedef  BOOL ( *DLL_rmSetMixerKeyNotifyMessages)(DWORD MixerKeyDownMsg,DWORD MixerKeyUpMsg);

//GPIO control
typedef  int  ( *DLL_rmSetPin2GPO)(DWORD Pins);  //pins exclude MatrixKey pins
typedef  int  ( *DLL_rmSetPinGPOLevel)(DWORD Levels);//pins exclude MatrixKey pins

//I2C control
typedef  BOOL ( *DLL_rmSetI2CGenericNotifyMessages)(DWORD MCUGenericMsg);
typedef  BOOL ( *DLL_rmSetI2CDeviceAddress)(BYTE address);
typedef  BOOL ( *DLL_rmWriteDataIntoI2C)(USHORT address,BYTE *data,USHORT len);
typedef  BOOL ( *DLL_rmReadDataFromI2C)(USHORT address,BYTE *data,USHORT len);

//general r/w
typedef  BOOL ( *DLL_rmSetReport)(BYTE *Report,BYTE Len);
typedef  BOOL ( *DLL_rmSetGenericNotifyMessages)(DWORD GeneralMsg);


RJ1102SDLL::RJ1102SDLL()
{
	m_DllHandle=NULL;
	m_PID=ImOpen=ReadReady=WaitAction=0;
	rmDeviceStart=NULL;
	rmIsDevicePresent=NULL;
	DeviceClose=NULL;
	rmHandleDeviceChangeNotification=NULL;
	rmSetPin2GPO=NULL;
	rmSetPinGPOLevel=NULL;
	rmSetI2CGenericNotifyMessages=NULL;
	rmSetI2CDeviceAddress=NULL;
	rmWriteDataIntoI2C=NULL;
	rmReadDataFromI2C=NULL;
	rmSetReport=NULL;
	rmSetGenericNotifyMessages=NULL;
	MyAddress=0;
	DeviceChecks=Written=0;
	InitializeCriticalSection(&m_CriticalSection);
	mR_PRE=mR_ATT=mR_WRT=0;
}

RJ1102SDLL::~RJ1102SDLL()
{
	CloseDLL();
}


BOOL RJ1102SDLL::OpenDLL(UINT PID)
{
	if(PID && PID!=m_PID) CloseDLL();
	if(m_DllHandle) return TRUE;

	char buf[256];
	sprintf(buf,"%s\\RJ1102S.dll",myDIR);
	m_DllHandle= LoadLibrary(buf);
	if(!m_DllHandle) return FALSE;

	if(!(rmDeviceStart=(DLL_rmDeviceStart)GetProcAddress(m_DllHandle,"rmDeviceStart"))) goto err;
	if(!(rmIsDevicePresent=(DLL_rmIsDevicePresent)GetProcAddress(m_DllHandle,"rmIsDevicePresent"))) goto err;
	if(!(DeviceClose=(DLL_rmDeviceClose)GetProcAddress(m_DllHandle,"rmDeviceClose"))) goto err;
	if(!(rmHandleDeviceChangeNotification =(DLL_rmHandleDeviceChangeNotification )GetProcAddress(m_DllHandle,"rmHandleDeviceChangeNotification"))) goto err;
	if(!(rmSetPin2GPO=(DLL_rmSetPin2GPO)GetProcAddress(m_DllHandle,"rmSetPin2GPO"))) goto err;
	if(!(rmSetPinGPOLevel=(DLL_rmSetPinGPOLevel)GetProcAddress(m_DllHandle,"rmSetPinGPOLevel"))) goto err;
	
	if(!(rmSetI2CGenericNotifyMessages=(DLL_rmSetI2CGenericNotifyMessages)GetProcAddress(m_DllHandle,"rmSetI2CGenericNotifyMessages"))) goto err;
	if(!(rmSetI2CDeviceAddress=(DLL_rmSetI2CDeviceAddress)GetProcAddress(m_DllHandle,"rmSetI2CDeviceAddress"))) goto err;
	if(!(rmWriteDataIntoI2C=(DLL_rmWriteDataIntoI2C)GetProcAddress(m_DllHandle,"rmWriteDataIntoI2C"))) goto err;
	if(!(rmReadDataFromI2C=(DLL_rmReadDataFromI2C)GetProcAddress(m_DllHandle,"rmReadDataFromI2C"))) goto err;
	if(!(rmSetReport=(DLL_rmSetReport)GetProcAddress(m_DllHandle,"rmSetReport"))) goto err;
	if(!(rmSetGenericNotifyMessages=(DLL_rmSetGenericNotifyMessages)GetProcAddress(m_DllHandle,"rmSetGenericNotifyMessages"))) goto err;

	if(!PID) PID=0x0132;
	rmDeviceStart(0x0D8C,PID,myPara.msgHWND,NULL,myPara.msgARRIVAL,myPara.msgREMOVED);
	m_PID=PID;
	return TRUE;
err:
	CloseDLL();
	return FALSE;
}

BOOL RJ1102SDLL::rmDeviceClose() //dummy, the CMedia.CM_HID.DLL.rmDeviceClose is corrupt
{
	ImOpen=0;
	MyAddress=0;
	return FALSE;
}

void RJ1102SDLL::CloseDLL()
{
	if(m_DllHandle)
		{
		rmDeviceClose();
		FreeLibrary(m_DllHandle);
		m_DllHandle=NULL;
		}
	else ImOpen=0;
}

BOOL RJ1102SDLL::OpenDevice(UINT PID)
{
	if(!m_DllHandle) OpenDLL(PID);
	if(!m_DllHandle) return FALSE;
	ImOpen=rmIsDevicePresent();
	if(!ImOpen) return FALSE;

	ReadReady=0;
	memset(&mCalib,0,sizeof(CALIBRATION_DATA));

	//Set RX/DX switch and ATT and EEprom WriteProtector
	WRT_BIT=0x08; //eeprom write enable 
	PRE_BIT=0x10; //preamplifier 
	ATT_BIT=0x80; //antennuator
	rmSetPin2GPO(ATT_BIT|PRE_BIT|WRT_BIT);	//set as Output direction
	rmSetPinGPOLevel(mR_PRE|mR_ATT|mR_WRT);	//set the state of the current out level

	UBYTE	buf[16];
	ReadEE(EEPROM_DEVICE_ID,buf,2);	//check prefixes (NT or BO)
	if(buf[0]!='C' || buf[1]!='M') return FALSE;

	ReadReady=1;
	ReadEE(EEPROM_CALIBRATION_SET,(UBYTE*)&mCalib,sizeof(CALIBRATION_DATA));	//read calibration data set
	if(mCalib.SerialPrefix[0]=='B')  NTIDRB32=0;	//RadioJet 1102S
	else  NTIDRB32=1;	//DiRaWave 0032U
	
	return TRUE;;
}

int RJ1102SDLL::CheckDevice() 
{ 
	if(ImOpen && m_PID)
		{
		DeviceChecks++;
		if(DeviceChecks==1) 
			{
			return DEVICE_FIRST_ARIVAL;
			}
		}
	if(ImOpen)
		{
		ImOpen=rmIsDevicePresent();
		if(!ImOpen) 
			{
			return DEVICE_NOT_FOUND;
			}
		else return DEVICE_READY; 
		}

	myCtrl.CSi570Driver::Open(m_PID);
	if(ImOpen) return DEVICE_NOW_READY;
	return DEVICE_NOT_FOUND;
} 

void RJ1102SDLL::SetGPIOLevel() 
{
	if(ImOpen) rmSetPinGPOLevel(mR_PRE|mR_ATT|mR_WRT);
}

BOOL RJ1102SDLL::SetI2CDeviceAddress(BYTE address)
{
 int waitmax=20;

waitloop:
	if(WaitAction && address!=WaitAction && waitmax)
		{
		waitmax--; Sleep(100);
		goto waitloop;
		}

	if(MyAddress!=address)
		{
		MyAddress=address;
		return rmSetI2CDeviceAddress(address);
		}
	return TRUE;
}

//the EEprom I2C-Address is 0xA0
BOOL RJ1102SDLL::ReadEE(USHORT address,BYTE *data,USHORT len) 
{
	if(!ImOpen) return FALSE;
	EnterCriticalSection(&m_CriticalSection);
	SetI2CDeviceAddress(0xA0);
	WaitAction=0xA0; //newer set if this is in action
	BOOL A=rmReadDataFromI2C(address,data,len); 
	WaitAction=0;
	LeaveCriticalSection(&m_CriticalSection);
	return A;
}
