// Radio.h: interface for the CDongleCtrl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(RJ10_DLL_1102)
#define RJ10_DLL_1102

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma pack(push)
#pragma pack(1)

typedef  BOOL (*DLL_rmDeviceStart)(WORD wVID, WORD wPID,HWND hWnd,void* pHIDCallbackProc, DWORD DeviceAddedMsg,DWORD DeviceRemovedMsg);
typedef  BOOL (*DLL_rmIsDevicePresent)(void);
typedef  BOOL (*DLL_rmDeviceClose)(void);
typedef  void (*DLL_rmHandleDeviceChangeNotification)(DWORD wParam, DWORD lParam);
typedef  BOOL (*DLL_rmSetMixerKeyNotifyMessages)(DWORD MixerKeyDownMsg,DWORD MixerKeyUpMsg);
//GPIO control
typedef  int  (*DLL_rmSetPin2GPO)(DWORD Pins);		
typedef  int  (*DLL_rmSetPinGPOLevel)(DWORD Levels);

//I2C control
typedef  BOOL (*DLL_rmSetI2CGenericNotifyMessages)(DWORD MCUGenericMsg);
typedef  BOOL (*DLL_rmSetI2CDeviceAddress)(BYTE address);
typedef  BOOL (*DLL_rmWriteDataIntoI2C)(USHORT address,BYTE *data,USHORT len);
typedef  BOOL (*DLL_rmReadDataFromI2C)(USHORT address,BYTE *data,USHORT len);

//general r/w
typedef  BOOL (*DLL_rmSetReport)(BYTE *Report,BYTE Len);
typedef  BOOL (*DLL_rmSetGenericNotifyMessages)(DWORD GeneralMsg);


#define RJ10_POW_2_16		65536
#define RJ10_POW_2_24		16777216
#define RJ10_POW_2_28		268435456

#define RJ10_FDCO_MAX		 5670.0 //5670.0 //5670.0         //MHz
#define RJ10_FDCO_MIN		 4850.0 //4850.0 //4850.0         //MHz

#define RJ10_VID	(unsigned int)0x0D8C	// Vendor ID
#define RJ10_PID	(unsigned int)0x0132	// Produkt ID of the RadioJet 1102S Device

#define DEVICE_NOT_READY			0		//device now disconnected or OFF (if it was connected)
#define DEVICE_READY				1		//device found and OK
#define DEVICE_NOW_READY			2		//device is now ready for new connection 
#define DEVICE_NOT_FOUND			-1		//device not connected and not found 
#define DEVICE_IS_WAITING			-99		//device is waiting for DEVICE_CHANGE 
#define DEVICE_FIRST_ARIVAL			-98		//if the device is activated, then the first times returns DEVICE_FIRST_ARIVAL

#define EEPROM_DEVICE_ID			0x00	// char[2] CM
#define EEPROM_CALIBRATION_SET		0x40	//EEprom read positon of the relevates data for RadioJet calibration


typedef struct	{
				short   AudioCorrecture;	//40 the audio chip own individual cristall clock difference offset to correct size of fax or sstv slant
				short	FreqCorrecture;		//42 the receiver chip own individual frequency offset
				short   ScaleCorrecture;	//44 the chip own individual frq scaling distance (difference from Low-Frq to Hi-Frq in Hz)
				short   DxFactor;			//46 the DXFactor of the DX-Channel
				short	StartUpFrq;			//48 the Si570 StartUp Frequency type type 0 or 1=15.0  2=56.32 
				time_t	LastBurning;		//4A (Internet Security) normal is 32 bit Date
				UBYTE	SerialPrefix[2];	//4E (Internet Security) two prefixes (NT or BO) or, or, or,...4A
				} CALIBRATION_DATA;

class RJ1102SDLL  
{
public:

	RJ1102SDLL();
	virtual ~RJ1102SDLL();

	BOOL		OpenDLL(UINT PID=RJ10_PID);
	void		CloseDLL();
	BOOL		OpenDevice(UINT PID);
	int			CheckDevice();

	void		SetGPIOLevel();
	BOOL		rmDeviceClose();
	BOOL		SetI2CDeviceAddress(BYTE address);
	BOOL		ReadEE(USHORT address,BYTE *data,USHORT len); 
	UINT64		currentRfreq,currentFreq;
	UBYTE		last_n1,last_hsdiv,n1,hsdiv;
    double		rfreq,fxtal_device,fxtal,RJ10_OUT_START_UP,corfacDiv;

	CALIBRATION_DATA mCalib;

	UBYTE		mR_PRE,mR_ATT,mR_WRT;	
	BYTE		MyAddress;
	UINT		m_PID;
	int			ImOpen,ReadReady,BurnerWaitTime,WaitAction,DeviceChecks,Written;
	HINSTANCE	m_DllHandle;

	DLL_rmDeviceStart					rmDeviceStart;
	DLL_rmIsDevicePresent				rmIsDevicePresent;
	DLL_rmDeviceClose					DeviceClose;
	DLL_rmHandleDeviceChangeNotification rmHandleDeviceChangeNotification;
	DLL_rmSetPin2GPO					rmSetPin2GPO;
	DLL_rmSetPinGPOLevel				rmSetPinGPOLevel;
	DLL_rmSetI2CGenericNotifyMessages	rmSetI2CGenericNotifyMessages;
	DLL_rmSetI2CDeviceAddress			rmSetI2CDeviceAddress;
	DLL_rmWriteDataIntoI2C				rmWriteDataIntoI2C;
	DLL_rmReadDataFromI2C				rmReadDataFromI2C;
	DLL_rmSetReport						rmSetReport;
	DLL_rmSetGenericNotifyMessages		rmSetGenericNotifyMessages;

	CRITICAL_SECTION m_CriticalSection;	//for keeping threads from stomping on each other
};

#pragma pack(pop)
#endif // !defined(DONGLE_CTRL)